(******************************************************************************)
(* TopSpeed compatible file i/o                                               *)
(* (C) 2009..2010 Steffen Solyga <solyga@absinth.net>                         *)
(******************************************************************************)
DEFINITION MODULE FIO;

FROM SYSTEM IMPORT BYTE;

TYPE
  File = CARDINAL;
  PathStr  = ARRAY [ 0 .. 255 ] OF CHAR;

CONST
  noFile = MAX( CARDINAL );

  StandardInput  = 0;
  StandardOutput = 1;
  ErrorOutput    = 2;

  resOk = 0;

(* controllers *)
VAR
  IOcheck : BOOLEAN;	(* if TRUE errors terminate program with report *)
  ChopOff : BOOLEAN;
  PrefixChar : CHAR;

CONST
  defIOcheck = TRUE;
  defChopOff = FALSE;
  defPrefixChar = ' ';

(* additional return values *)
VAR
  OK : BOOLEAN;	(* read & write: operation succeeded, see interfaces *)
  EOF: BOOLEAN;	(* read only: attempt to read beyond last byte (which isn't OK)
                   -- as far as detectable *)

(* Differences to TopSpeed's implementation:
 * 1. In contrast to the TopSpeed implementation, all open array parameter
 *    procs check the array size; the number of bytes requested is the minimum
 *    of this size and the explicit count (if any).
 * 2. TopSpeed doesn't use VAR for open arrays which should stay constant
 *    (WrBin, WrStr), but they switch open array copying off. Mocka doesn't
 *    have this option, so I'm using VAR for speedup.
 * 3. The usage of OK and EOF differ between JPI and TS 3.10, and I consider
 *    some cases as buggy (for instance that RdItem() returns OK on i/o error).
 *    My settings are explained with the interfaces.
 *)

(* result of latest call (IOcheck = FALSE) *)
PROCEDURE IOresult(): CARDINAL;

PROCEDURE Exists(   name: ARRAY OF CHAR ): BOOLEAN;
PROCEDURE Erase(    name: ARRAY OF CHAR );
PROCEDURE Open(     name: ARRAY OF CHAR ): File;
PROCEDURE OpenRead( name: ARRAY OF CHAR ): File;
PROCEDURE Create(   name: ARRAY OF CHAR ): File;
PROCEDURE Append(   name: ARRAY OF CHAR ): File;
PROCEDURE Close(        f: File );
PROCEDURE Size(         f: File ): LONGCARD;
PROCEDURE GetPos(       f: File ): LONGCARD;
PROCEDURE Seek(         f: File; p: LONGCARD );
PROCEDURE AssignBuffer( f: File; VAR b: ARRAY OF BYTE );

(* OK  = "exactly as many bytes written as requested" (see 1.)                *)
PROCEDURE WrBin(     f: File; VAR b: ARRAY OF BYTE; n: CARDINAL );
PROCEDURE WrChar(    f: File; c: CHAR );
PROCEDURE WrStr(     f: File; VAR s: ARRAY OF CHAR );
PROCEDURE WrCharRep( f: File; c: CHAR; n: CARDINAL );
PROCEDURE WrLn(      f: File );
PROCEDURE WrStrAdj(  f: File; VAR s: ARRAY OF CHAR; l: INTEGER );
PROCEDURE WrLngCard( f: File; x: LONGCARD;  l: INTEGER );
PROCEDURE WrCard(    f: File; x: CARDINAL;  l: INTEGER );
PROCEDURE WrShtCard( f: File; x: SHORTCARD; l: INTEGER );
PROCEDURE WrLngHex(  f: File; x: LONGCARD;  l: INTEGER );
PROCEDURE WrHex(     f: File; x: CARDINAL;  l: INTEGER );
PROCEDURE WrShtHex(  f: File; x: SHORTCARD; l: INTEGER );
PROCEDURE WrLngInt(  f: File; x: LONGINT;   l: INTEGER );
PROCEDURE WrInt(     f: File; x: INTEGER;   l: INTEGER );
PROCEDURE WrShtInt(  f: File; x: SHORTINT;  l: INTEGER );


(* ret = "number of bytes read" (0 if IOresult # resOk)                       *)
(* OK  = ( n = 0 ) OR ( ret > 0 )                                             *)
(* EOF = ( n > 0 ) & ( ret = 0 )                                              *)
PROCEDURE RdBin( f: File; VAR b: ARRAY OF BYTE; n: CARDINAL ): CARDINAL;

(* all other read procedures set OK and EOF as follows:                       *)
(* OK  = "operation successful"                                               *)
(* EOF = "attempt to read beyond last byte" (even when read ahead)            *)
PROCEDURE RdChar( f: File ): CHAR;

TYPE
  TIsSeparator = PROCEDURE ( CHAR ): BOOLEAN;
VAR
  isSep: TIsSeparator;

PROCEDURE RdStr( f: File; VAR s: ARRAY OF CHAR );
PROCEDURE RdItem( f: File; VAR s: ARRAY OF CHAR );

PROCEDURE RdLngCard( f: File ): LONGCARD;
PROCEDURE RdCard(    f: File ): CARDINAL;
PROCEDURE RdShtCard( f: File ): SHORTCARD;
PROCEDURE RdLngHex(  f: File ): LONGCARD;
PROCEDURE RdHex(     f: File ): CARDINAL;
PROCEDURE RdShtHex(  f: File ): SHORTCARD;
PROCEDURE RdLngInt(  f: File ): LONGINT;
PROCEDURE RdInt(     f: File ): INTEGER;
PROCEDURE RdShtInt(  f: File ): SHORTINT;


END FIO.
