(******************************************************************************)
(* Copyright (c) 1988 by GMD Karlruhe, Germany				      *)
(* Gesellschaft fuer Mathematik und Datenverarbeitung			      *)
(* (German National Research Center for Computer Science)		      *)
(* Forschungsstelle fuer Programmstrukturen an Universitaet Karlsruhe	      *)
(* All rights reserved.							      *)
(******************************************************************************)

(* for UNIX only *)
FOREIGN MODULE SysLib;

   FROM SYSTEM IMPORT ADDRESS;

   TYPE
      SIGNED    = INTEGER;
      UNSIGNED  = CARDINAL;

(* commented changes solyga@beast 2009-11-11 *)
      HUGECARD =
         RECORD
            l: LONGCARD;
            h: LONGCARD;
         END;
      HUGEINT  =
         RECORD
            l: LONGCARD;
            h: LONGINT;
         END;

      inoT      = LONGCARD;  (* ok *)
      offT      = LONGINT;   (* ok *)
      devT      = HUGECARD;  (* was SHORTCARD; *)
      timeT     = LONGINT;   (* ok *)
      modeT	= LONGCARD;  (* was SHORTCARD; *)
      uidT	= LONGCARD;  (* was SHORTCARD; *)
      gidT	= LONGCARD;  (* was SHORTCARD; *)
      nlinkT	= LONGCARD;  (* was SHORTCARD; *)
      clockT	= LONGINT;

(*    umodeT	= SHORTCARD; *)(* ??? *)

(* original SysLib.Stat had 64 bytes, my libc has 88 bytes (stat64)... *)
(* see /usr/src/linux/include/asm/stat.h *)
      Stat =
         RECORD
            stDev    : devT;       (* changed size *)
            pad1     : SHORTCARD;  (* ok *)
	    stIno    : inoT;       (* ok *)
            stMode   : modeT;      (* changed size *)
            stNlink  : nlinkT;     (* changed size *)
            stUid    : uidT;       (* changed size *)
            stGid    : gidT;       (* changed size *)
            stRdev   : devT;       (* changed size *)
            pad2     : SHORTCARD;  (* ok *)
	    stSize   : offT;       (* ok *)
            stBlksize: LONGINT;    (* was LONGCARD; *)
            stBlocks : LONGINT;    (* was LONGCARD; *)
            stAtime  : timeT;      (* ok *)
            unused1  : LONGINT;    (* was LONGCARD; *)
            stMtime  : timeT;      (* ok *)
            unused2  : LONGINT;    (* was LONGCARD; *)
            stCtime  : timeT;      (* ok *)
            unused3  : LONGINT;    (* was LONGCARD; *)
            unused4  : LONGCARD;   (* ok *)
            unused5 :  LONGCARD;   (* ok *)
         END;

      tms =
	 RECORD
	    utime  : clockT;  
	    stime  : clockT;
	    cutime : clockT;
	    cstime : clockT;
	 END;

   CONST

      (* flags for open *)

      oTRUNC   = 01000B;    (* open with truncation *)
      oAPPEND  = 02000B;    (* append, i.e writes at the end *)
      oRDWR    =    02B;    (* open for reading and writing *)
      oWRONLY  =    01B;    (* open for writing only *)
      oRDONLY  =     0B;    (* open for reading only *)

      (* file access permisson flags (for create and umask) *)

      pXUSID   = 04000B;    (* set user ID on execution *)
      pXGRID   = 02000B;    (* set group ID on execution *)
      pSTEXT   = 01000B;    (* save text image after execution *)
      pROWNER  =  0400B;    (* read by owner *)
      pWOWNER  =  0200B;    (* write by owner *)
      pXOWNER  =  0100B;    (* execute by owner *)
      pRGROUP  =   040B;    (* read by group *)
      pWGROUP  =   020B;    (* write by group *)
      pXGROUP  =   010B;    (* execute by group *)
      pROTHERS =    04B;    (* read by others *)
      pWOTHERS =    02B;    (* write by others *)
      pXOTHERS =    01B;    (* execute by others *)
      pEMPTY   =     0B;    (* no flag set *)
    
      (* file access check flags (for access) *)
 
      cREAD    = 04H;       (* check if readable *)
      cWRITE   = 02H;       (* check if writable *)
      cEXEC    = 01H;       (* check if executable *)
      cEXISTS  =  0H;       (* check existance *)

      (* added: offset references (for lseek) *)

      rSET = 0;
      rCUR = 1;
      rEND = 2;


   PROCEDURE umask (cmask : modeT) : modeT;  (* changed *)

   PROCEDURE access (path  : ADDRESS; amode : SIGNED) : SIGNED;

   PROCEDURE creat (path  : ADDRESS; cmode : SIGNED) : SIGNED;

   PROCEDURE open (path : ADDRESS; oflag : SIGNED) : SIGNED;

   PROCEDURE close (fildes : SIGNED) : SIGNED;

   PROCEDURE unlink (path : ADDRESS) : SIGNED;

   PROCEDURE read (fildes : SIGNED; buf : ADDRESS; nbyte : UNSIGNED) : SIGNED;

   PROCEDURE write (fildes : SIGNED; buf : ADDRESS; nbyte : UNSIGNED) : SIGNED;

   (* added 2009-11-11 *)
   PROCEDURE lseek (fildes : SIGNED; offset: offT; whence: SIGNED) : offT;

   PROCEDURE sbrk (incr : SIGNED): ADDRESS;

   PROCEDURE malloc (size : UNSIGNED) : ADDRESS;

   PROCEDURE free (ptr : ADDRESS);
   PROCEDURE stat (path: ADDRESS; VAR buf: Stat) : SIGNED;

   PROCEDURE fstat (fd: SIGNED  ; VAR buf: Stat) : SIGNED;
   PROCEDURE time (VAR t : INTEGER);

   PROCEDURE times (VAR buffer: tms);

   PROCEDURE system (string : ADDRESS) : SIGNED;

   PROCEDURE exit (n: SIGNED);

   PROCEDURE abort ();

END SysLib.
